/* Copyright (c)2012 Sky Co., LTD. All rights reserved. */

#import <Foundation/Foundation.h>
#import <Collaboration/Collaboration.h>

/**
 * ユーザー名やドメイン名等、ユーザーの識別情報を扱うための機能を提供します。
 * @since   SKYSEA8.0
 * @author  Morio Tanioku
 * @date    2012/08/06
 */
@interface IdentityUtils : NSObject {
    CBUserIdentity*     _userIdentity;      /**< ユーザー情報 */
    CBGroupIdentity*    _groupIdentity;     /**< グループ情報 */
    uint32_t            _sessionID;         /**< セキュリティセッションID */
    Boolean             _isActiveConsole;   /**< コンソールユーザーかどうか */
    Boolean             _isLoggedIn;        /**< ログインしているかどうか */
}

@property(assign, readwrite) uint32_t sessionID;
@property(assign, readwrite) Boolean isActiveConsole;
@property(assign, readwrite) Boolean isLoggedIn;

/**
 * コンソールユーザーの情報で構築します。
 * @return  コンソールユーザーの情報で初期化したインスタンスを返します。
 */
+ (id)identityForConsoleUser;
+ (id)identityForConsoleUserFromCache;

/**
 * プロセスの実行ユーザーの情報で構築します。
 * @return  プロセスの実行ユーザーの情報で初期化したインスタンスを返します。
 */
+ (id)identityForCurrentUser;
+ (id)identityForCurrentUserFromCache;

/**
 * コンソールユーザーの一覧を取得します。
 * @return  コンソールユーザーの情報で構築したIdentityUtilsインスタンス群を返します。
 */
+ (NSArray*)consoleUserIdentities;

/**
 * 指定されたユーザーの情報で初期化します。
 * @param   [in]    uid ユーザーID
 * @return  自身を返します。
 */
- (id)initWithPosixUID:(uid_t)uid;

/**
 * 指定されたユーザーの情報で初期化します。
 * @param   [in]    userName    ユーザー名
 * @return  自身を返します。
 */
- (id)initWithUserName:(NSString*)userName;

/**
 * @brief   ユーザーIDを取得します
 * @return  ユーザーID
 * @attention   ユーザー情報が取得できていない場合（_userIdentityがnil）、正しい値が取得できません
 */
- (uid_t)posixUID;

/**
 * アカウント名を取得します。
 * @return  アカウント名を返します。
 */
- (NSString*)posixUserName;

/**
 * アカウントのフルネームを取得します。
 * @return  アカウントのフルネームを返します。
 */
- (NSString*)fullName;

/**
 * グループ名を取得します。
 * @return  グループ名を返します。
 */
- (NSString*)posixGroupName;

/**
 * ドメイン名を取得します。
 * @return  最初の"."の直前までを抽出したドメイン名を返します。
 */
- (NSString*)domainName;

/**
 * 指定された長いドメイン名から末端のドメイン名を取得します。
 * @param   [in]    longDomainName  "."で区切られたドメイン名
 * @return  最初の"."の直前までを抽出したドメイン名を返します。
 */
+ (NSString*)domainName:(NSString*)longDomainName;

/**
 * 長いドメイン名を取得します。
 * @return  長いドメイン名を返します。
 */
- (NSString*)longDomainName;

/**
 * ユーザープリンシパル名を取得します。
 * @return  ユーザー名 "@" ドメイン名 形式のプリンシパル名を返します。
 */
- (NSString*)userPrincipalName;

/**
 * ダウンレベルユーザー名を取得します。
 * @return  ドメイン名 "¥" ユーザー名 形式のダウンレベルユーザー名を返します。
 */
- (NSString*)downLevelUserName;

/**
 * 指定されたユーザー名とドメイン名からダウンレベルユーザー名を取得します。
 * @param   [in]    userName    ユーザー名
 * @param   [in]    domainName  ドメイン名
 * @return  ドメイン名 "¥" ユーザー名 形式のダウンレベルユーザー名を返します。
 */
+ (NSString*)downLevelUserName:(NSString*)userName domain:(NSString*)domainName;

/**
 * 指定されたユーザー名と長いドメイン名からダウンレベルユーザー名を取得します。
 * @param   [in]    userName        ユーザー名
 * @param   [in]    longDomainName  長いドメイン名
 * @return  ドメイン名 "¥" ユーザー名 形式のダウンレベルユーザー名を返します。
 */
+ (NSString*)downLevelUserName:(NSString*)userName longDomain:(NSString*)longDomainName;

/**
 * 同一ユーザーかどうかテストします。
 * @param   [in]    userName    ユーザー名
 * @retval  YES ユーザー名が一致する場合
 * @retval  NO  ユーザー名が一致しない場合
 */
- (Boolean)isSameUser:(NSString*)userName;

/**
 * 同一ユーザーかどうかテストします。
 * @param   [in]    userName    ユーザー名
 * @param   [in]    domainName  ドメイン名
 * @retval  YES ユーザー名およびドメイン名が一致する場合
 * @retval  NO  ユーザー名またはドメイン名が一致しない場合
 */
- (Boolean)isSameUser:(NSString*)userName domain:(NSString*)domainName;

/**
 * 同一ユーザーかどうかテストします。
 * @param   [in]    downlevelUserName       比較元
 * @param   [in]    downlevelUserNameRight  比較対象
 * @param   [in]    ignoreDomain            ドメインを無視するかどうか
 * @return  ユーザーが一致した場合はYES、一致しなかった場合はNOを返します。
 */
+ (Boolean)isSameUser:(NSString*)downlevelUserName right:(NSString*)downlevelUserNameRight ignoreDomain:(Boolean)ignoreDomain;

/**
 * ダウンレベルユーザー名からドメインを取り除きます。
 * @param   [in]    downlevelUserName   ダウンレベルユーザー名
 */
+ (NSString*)trimDomain:(NSString*)downlevelUserName;

@end

/* Copyright (c)2012 Sky Co., LTD. All rights reserved. */
