/* Copyright (c)2012 Sky Co., LTD. All rights reserved. */

#import <Foundation/Foundation.h>
#import "StringTable.h"

/**
 * プロセス情報を表します。
 * @since   SKYSEA8.2
 * @author  Morio Tanioku
 * @date    2012/03/22
 */
@interface ProcessInfo : NSObject {
    pid_t           _pid;               /**< プロセスID */
    NSString*       _applicationName;   /**< アプリケーション名 */
    NSDate*         _createdDate;       /**< インスタンスの作成日時 */
    uint32_t        _sessionID;         /**< セッションID */
    StringTable*    _stringTable;       /**< 文字列テーブル */
    
    /** アカウント名 */
    NSString* _userNameString;          /**< 文字列 */
    NSNumber* _userNameResourceID;      /**< リソースID */
    
    /** アカウントのフルネーム */
    NSString* _fullNameString;          /**< 文字列 */
    NSNumber* _fullNameResourceID;      /**< リソースID */

    /** ドメイン名 */
    NSString* _domainNameString;        /**< 文字列 */
    NSNumber* _domainNameResourceID;    /**< リソースID */
    
    /** 実行パス */
    NSString* _pathString;              /**< 文字列 */
    NSMutableArray* _pathArray;         /**< パス区切り文字で分割したリソースID一覧 */

    pid_t           _ppid;              /**< 親プロセスID */
}

@property(assign, readonly) pid_t pid;
@property(retain, readonly) NSString* applicationName;
@property(retain, readwrite) NSDate* createdDate;
@property(assign, readwrite) uint32_t sessionID;
@property(assign, readonly) pid_t ppid;

/**
 * 初期化します。
 * @param   [in]    pid プロセスID
 * @return  自身を返します。
 */
- (id)initWithPID:(pid_t)pid;

/**
 * 初期化します。
 * @param   [in]    pid     プロセスID
 * @param   [in]    table   文字列テーブル
 * @return  自身を返します。
 */
- (id)initWithPID:(pid_t)pid stringTable:(StringTable*)table;

/**
 * 指定されたプロセスの情報を収集します。
 * @param   [in]    pid プロセスID
 */
- (void)collect:(pid_t)pid;

/**
 * ユーザー名を取得します。
 * @return  ユーザー名を返します。
 */
- (NSString*)userName;

/**
 * ユーザーのフルネームを取得します。
 * @return  ユーザーのフルネームを返します。
 */
- (NSString*)fullName;

/**
 * ドメイン名を取得します。
 * @return  ドメイン名を返します。
 */
- (NSString*)domainName;

/**
 * 実行パスを取得します。
 * @return  実行パスを返します。
 */
- (NSString*)path;

/**
 * ユーザー名を設定します。
 * @param   [in]    userName    ユーザー名
 */
- (void)setUserName:(NSString*)userName;

/**
 * ユーザーのフルネームを設定します。
 * @param   [in]    fullName    ユーザーのフルネーム
 */
- (void)setFullName:(NSString*)fullName;

/**
 * ドメイン名を設定します。
 * @param   [in]    domainName  ドメイン名
 */
- (void)setDomainName:(NSString*)domainName;

/**
 * 実行パスを設定します。
 * @param   [in]    path    実行パス
 */
- (void)setPath:(NSString*)path;

/**
 * プロセスの起動引数を取得します。
 * @param   [in]    pid プロセスID
 * @return  起動引数
 */
+ (NSArray*)getArguments:(pid_t)pid;

/**
 * プロセスのバンドルIDを取得します。
 * @param   [in]    pid プロセスID
 * @return  バンドルID
 */
+ (NSString*)getBundleIdentifier:(pid_t)pid;

/**
 * プロセスのスレッド情報一覧を取得します。
 * @return  ThreadInfo のリスト
 */
+ (NSArray*)threadsInfomation;

/**
 * 指定したプロセス名のプロセスIDのリストを取得します。
 * @param   [in]    procname    プロセス名（最大16byte）
 * @return  プロセスID のリスト
 * @attention   プロセス名は大文字小文字を無視して比較します。
 *              17byte以上の比較は出来ません。
 */
+ (NSSet*)pidlistWithName:(const char*)procname;

@end

/* Copyright (c)2012 Sky Co., LTD. All rights reserved. */
