/* Copyright (c)2012 Sky Co., LTD. All rights reserved. */

#import <Foundation/Foundation.h>
#import "WaitableQueue.h"
#import "MessagingPackets.h"

#define UP_MESSAGING_SERVICE_FORMAT @"%@-Up"
#define DOWN_MESSAGING_SERVICE_FORMAT @"%@-Down"

/**
 * 分割されたデータを結合するための機能を提供します。
 * @since   SKYSEA8.0
 * @author  Morio Tanioku
 * @date    2012/10/03
 */
@interface CombinedMessage : MessagingPacket {
    int32_t         _totalSize;     /**< 結合後のサイズ */
    NSMutableData*  _combinedData;  /**< 結合中のデータ */
    int64_t         _createdTime;   /**< 作成日時 */
}

@property(assign, readonly) int32_t totalSize;
@property(readonly) NSData* combinedData;
@property(assign, readonly) int64_t createdTime;

/**
 * 初期化します。
 * @param   [in]    seqID   シーケンスID
 * @param   [in]    _pid    プロセスID
 * @param   [in]    size    結合後のサイズ
 * @return  自身を返します。
 */
- (id)initWithSequenceID:(int32_t)seqID pid:(pid_t)pid totalSize:(int32_t)size;

/**
 * データを追加します。
 * @param   [in]    data    追加するデータ
 */
- (void)appendData:(NSData*)data;

/**
 * 結合が完了したかどうかテストします。
 * @retval  YES 結合が完了した
 * @retval  NO  結合が完了していない
 */
- (Boolean)completed;

@end

/**
 * プロセス間通信のサーバー機能を提供します。
 * @since   SKYSEA8.0
 * @author  Morio Tanioku
 * @date    2012/10/03
 */
@interface MessagingService : NSObject {
    NSString*       _serviceName;   /**< サービス名(プロセス間通信のプロキシ名として使用) */
    WaitableQueue*  _upQueue;       /**< サーバー→クライアントへのデータを蓄えるためのキュー */
    NSConnection*   _upService;     /**< サーバー→クライアントへのキューにアクセスするためのコネクション */
    NSThread*       _upThread;      /**< サーバー→クライアントへのキューへの接続を待ち受けるスレッド */
    WaitableQueue*  _downQueue;     /**< クライアント→サーバーへのデータを蓄えるためのキュー */
    NSConnection*   _downService;   /**< クライアント→サーバーへのキューにアクセスするためのコネクション */
    NSThread*       _downThread;    /**< クライアント→サーバーへのキューへの接続を待ち受けるスレッド */
    NSThread*       _readerThread;  /**< クライアント→サーバーへのデータを読み出すためのスレッド */
    WaitableQueue*  _readableQueue; /**< クライアントからのデータを復号化したデータ群 */
}

/**
 * プロセス間通信のサーバー機能を開始します。
 * @param   [in]    name        サービス名
 * @param   [in]    threadName  内部スレッドに付ける名前
 * @return  自身を返します。
 */
- (id)initWithName:(NSString*)name threadName:(NSString*)threadName;

/**
 * クライアントから受信した全てのデータを取得し、内部バッファを空にします。
 * @return  保持していた全てのデータを返します。
 */
- (NSArray*)read;

/**
 * クライアントから受信した全てのデータを取得し、内部バッファを空にします。
 * @param   [in]    timeout データ取得のタイムアウト値(秒)
 * @param   [out]   buffer  保持している全てのデータを取得するためのバッファ
 * @retval  YES データが取得できた場合
 * @retval  NO  データが取得できなかった or タイムアウトした場合
 */
- (Boolean)readWithTimeout:(NSTimeInterval)timeout buffer:(NSArray**)buffer;

/**
 * クライアントにデータを渡します。
 * @param   [in]    message 追加したいデータ
 */
- (void)write:(id<NSCoding>)message;

@end
