/* Copyright (c)2012 Sky Co., LTD. All rights reserved. */

#import <Foundation/Foundation.h>

/**
 * Windows INIのようなセクション、キー、値から成る辞書を扱うための機能を提供します。
 * @since   SKYSEA8.0
 * @author  Morio Tanioku
 * @date    2012/10/06
 */
@interface Properties : NSObject {
    NSMutableDictionary* _keyValues;    /**< セクション、キー、値から成る辞書 */
}

/**
 * 初期化します。
 * @return  自身を返します。
 */
- (id)init;

/**
 * ファイルを基に初期化します。
 * @param   [in]    path    ファイルパス
 * @return  自身を返します。
 */
- (id)initWithContentsOfFile:(NSString*)path;

/**
 * Boolean値を取得します。
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 * @return  対応する文字列をBooleanに変換したものを返します。
 */
- (Boolean)getBooleanForSection:(NSString*)section forKey:(NSString*)key;
- (Boolean)getBooleanForSection:(NSString*)section forKey:(NSString*)key default:(Boolean)defValue;

/**
 * 日時を取得します。
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 * @return  対応する日時を返します。
 */
- (NSDate*)getDateForSection:(NSString*)section forKey:(NSString*)key;
- (NSDate*)getDateForSection:(NSString*)section forKey:(NSString*)key default:(NSDate*)defValue;

/**
 * double値を取得します。
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 * @return  対応する文字列をdoubleに変換したものを返します。
 */
- (double)getDoubleForSection:(NSString*)section forKey:(NSString*)key;
- (double)getDoubleForSection:(NSString*)section forKey:(NSString*)key default:(double)defValue;

/**
 * 文字列を取得します。
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 * @return  対応する文字列を返します。
 */
- (NSString*)getStringForSection:(NSString*)section forKey:(NSString*)key;
- (NSString*)getStringForSection:(NSString*)section forKey:(NSString*)key default:(NSString*)defValue;

/**
 * 32ビット整数値を取得します。
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 * @return  対応する文字列を32ビット整数値に変換したものを返します。
 */
- (int32_t)getIntForSection:(NSString*)section forKey:(NSString*)key;
- (int32_t)getIntForSection:(NSString*)section forKey:(NSString*)key default:(uint32_t)defValue;

/**
 * 64ビット整数値値を取得します。
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 * @return  対応する文字列を64ビット整数値に変換したものを返します。
 */
- (int64_t)getInt64ForSection:(NSString*)section forKey:(NSString*)key;
- (int64_t)getInt64ForSection:(NSString*)section forKey:(NSString*)key default:(int64_t)defValue;

/**
 * Boolean値を設定します。
 * @param   [in]    value   設定する値
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 */
- (void)putBoolean:(Boolean)value forSection:(NSString*)section forKey:(NSString*)key;

/**
 * 日時を設定します。
 * @param   [in]    value   設定する値
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 */
- (void)putDate:(NSDate*)value forSection:(NSString*)section forKey:(NSString*)key;

/**
 * double値を設定します。
 * @param   [in]    value   設定する値
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 */
- (void)putDouble:(double)value forSection:(NSString*)section forKey:(NSString*)key;

/**
 * 文字列を設定します。
 * @param   [in]    value   設定する値
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 */
- (void)putString:(NSString*)value forSection:(NSString*)section forKey:(NSString*)key;

/**
 * 32ビット整数値を設定します。
 * @param   [in]    value   設定する値
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 */
- (void)putInt:(int32_t)value forSection:(NSString*)section forKey:(NSString*)key;

/**
 * 64ビット整数値を設定します。
 * @param   [in]    value   設定する値
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 */
- (void)putInt64:(int64_t)value forSection:(NSString*)section forKey:(NSString*)key;

/**
 * セクション名を列挙するための列挙子を取得します。
 * @return  セクション名を列挙するための列挙子を返します。
 */
- (NSEnumerator*)sectionEnumerator;

/**
 * 全てのセクション名を配列で取得します。
 * @return  全てのセクション名が入っている配列を返します。
 */
- (NSArray*)allSectionNames;

/**
 * 指定されたセクションの、全てのキーと値を取得します。
 * @param   [in]    section セクション名
 * @return  セクション名に関連するキーと値の辞書を返します。
 */
- (NSDictionary*)allKeyValuesForSection:(NSString*)section;

/**
 * 全ての内容を破棄します。
 */
- (void)removeAllObjects;

/**
 * 指定されたセクションの内容を破棄します。
 * @param   [in]    section セクション名
 */
- (void)removeObjectForSection:(NSString*)section;

/**
 * 指定されたセクションおよびキーに一致する値を破棄します。
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 */
- (void)removeObjectForSection:(NSString*)section forKey:(NSString*)key;

/**
 * 指定されたセクションが存在するかどうかテストします。
 * @param   [in]    section セクション名
 * @retval  YES セクション名が存在している
 * @retval  NO  セクション名が存在していない
 */
- (Boolean)containsSection:(NSString*)section;

/**
 * 指定されたセクションおよびキーが存在するかどうかテストします。
 * @param   [in]    section セクション名
 * @param   [in]    key     キー名
 * @retval  YES セクション名が存在している
 * @retval  NO  セクション名が存在していない
 */
- (Boolean)containsSection:(NSString*)section forKey:(NSString*)key;

/**
 * このインスタンスの説明を取得します。
 * @return  説明を文字列で返します。
 */
- (NSString*)description;

/**
 * PList形式でファイルに書き込みます。
 * @param   [in]    path        ファイルパス
 * @param   [in]    atomically  アトミックに書き込むかどうか
 * @retval  YES 書き込みに成功した
 * @retval  NO  書き込みに失敗した
 */
- (Boolean)writeToFile:(NSString*)path;
- (Boolean)writeToFile:(NSString*)path atomically:(BOOL)atomically;

@end

/* Copyright (c)2012 Sky Co., LTD. All rights reserved. */
